/* eslint-disable import/unambiguous, import/no-commonjs */
module.exports = {
  parser: 'babel-eslint',
  parserOptions: {
    sourceType: 'module',
    ecmaFeatures: {
      experimentalDecorators: true,
      jsx: true
    }
  },
  plugins: ['babel', 'react', 'import'],
  env: {
    browser: true,
    node: true,
    es6: true
  },
  globals: {
    it: true,
    Promise: true,
    fetch: true,
    TimeoutID: true,
    IntervalID: true,
    AnimationFrameID: true
  },
  rules: {
    // getter 和 setter 要同时存在，否则就报错
    'accessor-pairs': 'error',
    // 在数组开括号后和闭括号前强制换行
    'array-bracket-newline': ['warn', { multiline: true }],
    // 禁止在括号内使用空格
    'array-bracket-spacing': ['error', 'never'],
    // 强制数组方法的回调函数中有 return 语句
    'array-callback-return': 'error',
    // 不强制要求箭头函数体使用大括号，如果单一的表达式，可以不需要大括号
    'arrow-body-style': 'off',
    // 不强制要求箭头函数的参数使用圆括号，当只有一个参数时允许省略圆括号
    'arrow-parens': ['error', 'as-needed'],
    // 强制要求箭头函数的箭头之前或之后有空格
    'arrow-spacing': 'error',
    // 把 var 语句看作是在块级作用域范围之内
    'block-scoped-var': 'warn',
    // 禁止在代码块中开括号前和闭括号后有空格
    'block-spacing': ['error', 'never'],
    // 不强制要求大括号风格
    'brace-style': 'off',
    // 不强制要求返回callback函数
    'callback-return': 'off',
    // 强制要求使用骆驼拼写法
    camelcase: 'error',
    // 不强制对注释的第一个字母大写
    'capitalized-comments': 'off',
    // 不强制类方法使用 this
    'class-methods-use-this': 'off',
    // 禁止使用拖尾逗号
    'comma-dangle': 'error',
    // 强制逗号后使用空格
    'comma-spacing': ['error', { before: false, after: true }],
    // 逗号风格: 要求逗号放在数组元素、对象属性或变量声明之后，且在同一行
    'comma-style': ['error', 'last'],
    // 限制圈复杂度，默认 30，如果太大，就建议拆分函数
    complexity: ['warn', 30],
    // 禁止在计算属性中使用空格
    'computed-property-spacing': ['error', 'never'],
    // 要求使用一致的 return 语句
    'consistent-return': ['error', { treatUndefinedAsUnspecified: true }],
    // 不要求一致的 This
    'consistent-this': 'off',
    // 构造函数中必须调用 super
    'constructor-super': 'error',
    // 不允许省略大括号
    curly: ['error', 'all'],
    // 不要求 Switch 语句中有 Default 分支
    'default-case': 'off',
    // 点号如果访问属性的时候，不允许换行
    'dot-location': ['error', 'property'],
    // 要求使用点号代替方括号来访问属性
    'dot-notation': ['error', { allowKeywords: true }],
    // 不要求文件末尾保留一行空行
    'eol-last': ['off', 'always'],
    // 要求使用 === 和 !==，跟 null 比较除外
    eqeqeq: ['error', 'always', { null: 'ignore' }],
    // 不强制 “for” 循环中更新子句的计数器朝着正确的方向移动
    'for-direction': 'off',
    // 禁止在函数名和开括号之间有空格
    'func-call-spacing': ['error', 'never'],
    // 不要求函数名与赋值给它们的变量名或属性名相匹配
    'func-name-matching': 'off',
    // 不要求命名的 function 表达式
    'func-names': ['warn', 'always'],
    // 不强制 function 声明或表达式的一致性
    'func-style': 'off',
    // 不强制 generator 函数中 * 号周围有空格
    'generator-star-spacing': 'off',
    // 不强制在模块顶部调用 require()
    'global-require': 'warn',
    // 需要约束 for-in，如果没有 hasOwnProperty 那么会提示 warning
    'guard-for-in': 'warn',
    // 不强制回调错误处理
    'handle-callback-err': 'off',
    // 不禁止使用指定的标识符
    'id-blacklist': 'off',
    // 不强制标识符的最大和最小长度
    'id-length': 'off',
    // 不要求标识符匹配特定的正则表达式
    'id-match': 'off',
    // 默认缩进 2 个空格
    indent: ['error', 2, { SwitchCase: 1 }],
    'indent-legacy': 'off',
    // 如果变量在声明的时候没有初始化，那么会提示 warning
    'init-declarations': 'warn',
    // 强制在 JSX 属性中使用一致的双引号
    'jsx-quotes': ['error', 'prefer-double'],
    // 强制在对象字面量的键和值之间使用一致的空格
    'key-spacing': ['error', { beforeColon: false, afterColon: true }],
    // 强制关键字周围使用空格
    'keyword-spacing': 'error',
    // 不强制行注释的位置，可以在上方或者侧方
    'line-comment-position': 'off',
    // 强制使用一致的换行符风格
    'linebreak-style': ['error', 'unix'],
    // 不强制注释周围有空行
    'lines-around-comment': 'off',
    // 强制块语句的最大可嵌套深度，默认是 5 超过了提示 warning
    'max-depth': ['warn', 5],
    // 强制行的最大长度 120 超过了提示 warning
    'max-len': ['warn', 120, 4, { ignoreUrls: true }],
    // 强制文件的最大行数 800 超过了提示 warning
    'max-lines': ['warn', 800],
    // 不强制要求回调函数最大嵌套深度
    'max-nested-callbacks': 'off',
    // 不限制函数定义中最大参数个数
    'max-params': 'off',
    // 限制函数块中的语句的最大数量，默认 100，超过了提示 warning
    'max-statements': ['warn', 100, { ignoreTopLevelFunctions: true }],
    // 强制每一行中所允许的最大语句数量
    'max-statements-per-line': 'error',
    // 不禁止在三元操作数中间换行
    'multiline-ternary': 'off',
    // 要求构造函数首字母大写
    'new-cap': 'error',
    // 要求调用无参构造函数时带括号
    'new-parens': 'warn',
    // 不要求方法链中每个调用都有一个换行符
    'newline-per-chained-call': 'off',
    // 使用 alert 的地方给出 warning
    'no-alert': 'warn',
    // 可以使用 Array 构造函数
    'no-array-constructor': 'off',
    // 可以在循环中出现 await
    'no-await-in-loop': 'off',
    // 可以使用按位操作符
    'no-bitwise': 'off',
    // 使用 Buffer 构造函数的时候给出 warning
    'no-buffer-constructor': 'warn',
    // 不允许使用 caller 或 callee
    'no-caller': 'error',
    // 允许在 case 或 default 子句中出现词法声明
    'no-case-declarations': 'off',
    // 允许在 catch 语句中遮盖变量，不考虑 ie8 了
    'no-catch-shadow': 'off',
    // 不允许修改类声明的变量
    'no-class-assign': 'error',
    // 禁止与 -0 进行比较，如果需要比较的话，用 Object.is(x, -0)
    'no-compare-neg-zero': 'error',
    // 禁止在条件语句中出现赋值操作符，例如 if (x = 0) 是不允许的
    'no-cond-assign': 'error',
    // 禁止在可能与比较操作符相混淆的地方使用箭头函数
    'no-confusing-arrow': ['error', { allowParens: true }],
    // 可以使用 console.xxx 的函数
    'no-console': 'off',
    // 不允许改变用const声明的变量
    'no-const-assign': 'error',
    // 禁止在条件中使用常量表达式，例如 if (false) { }
    'no-constant-condition': 'error',
    // 允许使用 continue
    'no-continue': 'off',
    // 禁止在正则表达式中使用控制字符
    'no-control-regex': 'error',
    // 禁止使用 debugger
    'no-debugger': 'error',
    // 禁止删除变量
    'no-delete-var': 'error',
    // 允许使用看起来像除法的正则表达式
    'no-div-regex': 'off',
    // 禁止在 function 定义中出现重复的参数
    'no-dupe-args': 'error',
    // 禁止类成员中有重复的名称
    'no-dupe-class-members': 'error',
    // 禁止在对象字面量中出现重复的键
    'no-dupe-keys': 'error',
    // 禁止重复 case 标签
    'no-duplicate-case': 'error',
    // 禁止重复导入
    'no-duplicate-imports': 'warn',
    // 允许在 else 前有 return
    'no-else-return': 'off',
    // 禁止空块语句
    'no-empty': 'error',
    // 禁止在正则表达式中出现空字符集
    'no-empty-character-class': 'error',
    // 禁止出现空函数
    'no-empty-function': 'error',
    // 禁止使用空解构模式
    'no-empty-pattern': 'error',
    // 允许与 null 进行比较
    'no-eq-null': 'off',
    // 禁止使用 eval
    'no-eval': 'error',
    // 禁止对 catch 子句中的异常重新赋值
    'no-ex-assign': 'error',
    // 禁止扩展原生对象
    'no-extend-native': 'error',
    // 禁止不必要的函数绑定
    'no-extra-bind': 'warn',
    // 禁止不必要的布尔类型转换
    'no-extra-boolean-cast': 'error',
    // 禁用不必要的标签
    'no-extra-label': 'error',
    // 禁止冗余的括号
    'no-extra-parens': ['error', 'functions'],
    // 禁用不必要的分号
    'no-extra-semi': 'error',
    // 禁止 case 语句落空
    'no-fallthrough': 'error',
    // 禁止浮点小数
    'no-floating-decimal': 'error',
    // 禁止对 function 声明重新赋值
    'no-func-assign': 'warn',
    // 禁止对原生对象或只读的全局对象进行赋值
    'no-global-assign': 'error',
    // 允许使用较短的符号实现类型转换
    'no-implicit-coercion': 'off',
    // 允许在全局范围使用变量和函数声明
    'no-implicit-globals': 'off',
    // 禁止隐式的eval()
    'no-implied-eval': 'error',
    // 允许使用内联注释
    'no-inline-comments': 'off',
    // 禁止在嵌套的语句块中出现变量或 function 声明
    'no-inner-declarations': 'error',
    // 禁止在 RegExp 构造函数中出现无效的正则表达式
    'no-invalid-regexp': 'error',
    // 允许 this 关键字在类或类对象之外出现
    'no-invalid-this': 'off',
    // 禁止不规则的空白
    'no-irregular-whitespace': 'error',
    // 允许使用迭代器
    'no-iterator': 'off',
    // 允许与变量同名的标签
    'no-label-var': 'off',
    // 允许标签语句
    'no-labels': 'off',
    // 允许不必要的嵌套块
    'no-lone-blocks': 'off',
    // 禁止 if 语句作为唯一语句出现在 else 语句块中
    'no-lonely-if': 'error',
    // 禁止循环中存在函数
    'no-loop-func': 'warn',
    // 允许使用魔术数字
    'no-magic-numbers': 'off',
    'no-mixed-operators': 'off',
    // 禁止 require 调用与普通变量声明混合使用
    'no-mixed-requires': 'off',
    // 禁止使用 空格 和 tab 混合缩进
    'no-mixed-spaces-and-tabs': 'error',
    // 禁止连续赋值
    'no-multi-assign': 'warn',
    // 禁止出现多个空格
    'no-multi-spaces': [
      'error',
      { ignoreEOLComments: true, exceptions: { Property: false } }
    ],
    // 禁止多行字符串
    'no-multi-str': 'warn',
    // 不允许多个空行
    'no-multiple-empty-lines': 'warn',
    // 允许否定表达式
    'no-negated-condition': 'off',
    // 禁止在表达式中否定左操作数
    'no-negated-in-lhs': 'error',
    // 允许使用嵌套的三元表达式
    'no-nested-ternary': 'off',
    // 禁止使用 new 以避免产生副作用
    'no-new': 'error',
    // 禁用Function构造函数
    'no-new-func': 'error',
    // 禁止使用 Object 构造函数
    'no-new-object': 'error',
    // 不允许 new require
    'no-new-require': 'error',
    // 禁止 Symbolnew 操作符和 new 一起使用
    'no-new-symbol': 'error',
    // 禁止原始包装实例
    'no-new-wrappers': 'error',
    // 禁止将全局对象当作函数进行调用
    'no-obj-calls': 'error',
    // 禁用八进制字面量
    'no-octal': 'error',
    // 禁止在字符串字面量中使用八进制转义序列
    'no-octal-escape': 'error',
    // 禁止对函数参数再赋值
    'no-param-reassign': ['warn', { props: true }],
    // 当使用 _dirname 和 _filename 时允许字符串拼接
    'no-path-concat': 'off',
    // 允许使用一元操作符 ++ 和 --
    'no-plusplus': 'off',
    // 允许使用 process.env
    'no-process-env': 'off',
    // 允许使用 process.exit()
    'no-process-exit': 'off',
    // 禁用__proto__
    'no-proto': 'error',
    // 允许使用 Object.prototypes 的内置属性
    'no-prototype-builtins': 'off',
    // 禁止重新声明变量
    'no-redeclare': 'error',
    // 禁止正则表达式字面量中出现多个空格
    'no-regex-spaces': 'error',
    // 允许使用特定的全局变量
    'no-restricted-globals': 'off',
    // 允许使用特定的 import
    'no-restricted-imports': 'off',
    // 允许使用 Node.js 模块
    'no-restricted-modules': 'off',
    // 允许使用对象的某些属性
    'no-restricted-properties': 'off',
    // 允许使用特定的语法
    'no-restricted-syntax': 'off',
    // 禁止在返回语句中赋值
    'no-return-assign': ['error', 'except-parens'],
    // 禁用不必要的 return await
    'no-return-await': 'error',
    // 禁用 Script URL
    'no-script-url': 'warn',
    // 禁止自身赋值
    'no-self-assign': 'error',
    // 禁止自身比较
    'no-self-compare': 'error',
    // 不允许使用逗号操作符
    'no-sequences': 'error',
    // 允许变量声明覆盖外层作用域的变量
    'no-shadow': 'off',
    // 允许关键字被遮蔽
    'no-shadow-restricted-names': 'off',
    // 禁止函数名和左括号之间有空格
    'no-spaced-func': 'error',
    // 禁用稀疏数组
    'no-sparse-arrays': 'error',
    // 允许使用 Node.js 中的同步方法
    'no-sync': 'off',
    // 禁用 tab
    'no-tabs': 'warn',
    // 禁止在常规字符串中出现模板字面量占位符语法
    'no-template-curly-in-string': 'off',
    // 允许使用三元操作符
    'no-ternary': 'off',
    // 禁止在构造函数在调用super()之前使用this或super
    'no-this-before-super': 'error',
    // 限制可以被抛出的异常，最好抛出 Error 类型
    'no-throw-literal': 'warn',
    // 禁用行尾空白
    'no-trailing-spaces': 'error',
    // 禁用未声明的变量
    'no-undef': 'error',
    // 不允许初始化变量值为 undefined
    'no-undef-init': 'warn',
    // 允许使用undefined变量
    'no-undefined': 'off',
    // 禁止标识符中有悬空下划线
    'no-underscore-dangle': 'warn',
    // 禁止使用令人困惑的多行表达式
    'no-unexpected-multiline': 'error',
    // 禁用一成不变的循环条件
    'no-unmodified-loop-condition': 'warn',
    // 禁止可以表达为更简单结构的三元操作符
    'no-unneeded-ternary': 'error',
    // 禁止在 return、throw、continue 和 break 语句后出现不可达代码
    'no-unreachable': 'error',
    // 禁止在 finally 语句块中出现控制流语句
    'no-unsafe-finally': 'error',
    // 禁止对关系运算符的左操作数使用否定操作符
    'no-unsafe-negation': 'error',
    // 禁止未使用过的表达式
    'no-unused-expressions': ['warn', { allowShortCircuit: true }],
    // 禁用未使用过的标签
    'no-unused-labels': 'error',
    // 禁止未使用过的变量, React 除外
    'no-unused-vars': ['error', { varsIgnorePattern: '^React$' }],
    // 禁止定义前使用
    'no-use-before-define': [
      'error',
      { variables: true, classes: false, functions: false }
    ],
    // 禁用不必要的 .call() 和 .apply()
    'no-useless-call': 'warn',
    // 禁止在对象中使用不必要的计算属性
    'no-useless-computed-key': 'error',
    // 禁止没有必要的字符拼接，例如："a" + "b"
    'no-useless-concat': 'warn',
    // 禁用不必要的构造函数
    'no-useless-constructor': 'error',
    // 禁用不必要的转义
    'no-useless-escape': 'error',
    // 禁止在 import 和 export 和解构赋值时将引用重命名为相同的名字
    'no-useless-rename': 'error',
    // 禁止多余的 return 语句
    'no-useless-return': 'error',
    // 要求使用 let 或 const 而不是 var
    'no-var': 'error',
    // 禁止使用void操作符
    'no-void': 'error',
    // 允许使用警告注释
    'no-warning-comments': 'off',
    // 禁止属性前有空白
    'no-whitespace-before-property': 'error',
    // 禁用 with 语句
    'no-with': 'error',
    // 不强制单个语句的位置
    'nonblock-statement-body-position': 'off',
    // 强制在花括号内使用一致的换行符
    'object-curly-newline': ['error', { consistent: true, multiline: true }],
    // 不强制在花括号中使用一致的空格
    'object-curly-spacing': 'off',
    // 不强制将对象的属性放在不同的行上
    'object-property-newline': 'off',
    // 保证对象字面量的简写或非简写一致性，但尽可能的全部使用简写
    'object-shorthand': ['off', 'consistent-as-needed'],
    // 强制函数中的变量在一起声明或分开声明
    'one-var': ['error', 'never'],
    // 强制在变量声明周围换行
    'one-var-declaration-per-line': 'error',
    'operator-assignment': 'off',
    'operator-linebreak': 'off',
    'padded-blocks': 'off',
    'padding-line-between-statements': 'off',
    // 要求使用箭头函数作为回调
    'prefer-arrow-callback': 'warn',
    // 建议使用const
    'prefer-const': 'error',
    'prefer-destructuring': 'off',
    // 禁用 parseInt() 和 Number.parseInt()，使用二进制，八进制和十六进制字面量
    'prefer-numeric-literals': 'error',
    // 要求使用 Error 对象作为 Promise 拒绝的原因
    'prefer-promise-reject-errors': 'warn',
    'prefer-reflect': 'off',
    // 建议使用剩余参数代替 arguments
    'prefer-rest-params': 'error',
    // 建议使用扩展运算符而非.apply()
    'prefer-spread': 'warn',
    'prefer-template': 'off',
    'quote-props': 'off',
    // 强制使用一致的双引号
    quotes: ['error', 'single'],
    // 使用 parseInt 的时候要求必须有基数
    radix: 'error',
    // 禁止使用不带 await 表达式的 async 函数
    'require-await': 'warn',
    'require-jsdoc': 'off',
    // 禁用函数内没有yield的 generator 函数
    'require-yield': 'error',
    // 强制剩余和扩展运算符及其表达式之间有空格
    'rest-spread-spacing': ['error', 'never'],
    // 要求使用分号代替 ASI
    semi: ['error', 'always'],
    // 禁止分号前后有空格
    'semi-spacing': 'error',
    // 强制分号出现在行尾
    'semi-style': ['error', 'last'],
    'sort-imports': 'off',
    'sort-keys': 'off',
    'sort-vars': 'off',
    // 语句块之前必须总是至少有一个前置空格
    'space-before-blocks': ['error', 'always'],
    'space-before-function-paren': 'off',
    // 禁止或强制圆括号内的空格
    'space-in-parens': ['error', 'never'],
    // 要求中缀操作符周围有空格
    'space-infix-ops': 'error',
    // 要求或禁止在一元操作符之前或之后存在空格
    'space-unary-ops': 'warn',
    // 要求或禁止在注释前有空白
    'spaced-comment': ['error', 'always'],
    strict: 'off',
    // 强制在 switch 的冒号左右有空格
    'switch-colon-spacing': ['error', { before: false, after: true }],
    // 要求 symbol 描述
    'symbol-description': 'warn',
    // 强制模板字符串中空格的使用
    'template-curly-spacing': ['error', 'never'],
    // 要求或禁止在模板标记和它们的字面量之间有空格
    'template-tag-spacing': ['error', 'never'],
    // 要求或禁止使用 Unicode 字节顺序标记
    'unicode-bom': 'warn',
    // 要求调用 isNaN()检查 NaN
    'use-isnan': 'error',
    'valid-jsdoc': 'off',
    // 强制 typeof 表达式与有效的字符串进行比较
    'valid-typeof': 'error',
    'vars-on-top': 'off',
    // 需要把立即执行的函数包裹起来
    'wrap-iife': ['error', 'any'],
    'wrap-regex': 'off',
    // 强制在 yield* 表达式中 * 周围使用空格
    'yield-star-spacing': 'off',
    // 警告Yoda条件
    yoda: 'warn',

    'babel/object-curly-spacing': 'off', // ['error', 'never'],

    // 强制 import 的文件必须存在
    'import/no-unresolved': 'error',
    // 禁止使用没有 export 的变量
    'import/named': 'error',
    // 如果没有导出 default，但是使用了，会报错
    'import/default': 'error',
    'import/namespace': 'off',
    'import/no-restricted-paths': 'off',
    // 禁止 import 绝对的文件路径
    'import/no-absolute-path': 'error',
    // 禁止 require 里面的内容包含表达式
    'import/no-dynamic-require': 'warn',
    'import/no-internal-modules': 'off',
    // 禁止 webpack 中 require 的语法
    'import/no-webpack-loader-syntax': 'warn',
    // 禁止重复 export 同名的变量
    'import/export': 'error',
    'import/no-named-as-default': 'warn',
    'import/no-named-as-default-member': 'error',
    'import/no-deprecated': 'error',
    'import/no-extraneous-dependencies': [
      'error',
      {
        devDependencies: true,
        optionalDependencies: true,
        peerDependencies: true
      }
    ],
    // 针对 export 的变量，禁止使用 let 或者 var
    'import/no-mutable-exports': 'error',
    'import/unambiguous': 'warn',
    'import/no-commonjs': 'error',
    'import/no-amd': 'error',
    // 进制 import Node.js 内置的模块
    'import/no-nodejs-modules': 'error',
    // import 必须是文件中的第一个语句
    'import/first': 'error',
    // 禁止 import 同名的文件
    'import/no-duplicates': 'error',
    'import/no-namespace': 'off',
    'import/extensions': [
      'error',
      'always',
      { js: 'never', jsx: 'never', es: 'never', mjs: 'never' }
    ],
    'import/order': [
      'error',
      {
        groups: [
          'builtin',
          'external',
          'internal',
          'parent',
          'sibling',
          'index'
        ]
      }
    ],
    'import/newline-after-import': 'error',
    'import/prefer-default-export': 'off',
    'import/max-dependencies': 'off',
    'import/no-unassigned-import': 'off',
    'import/no-named-default': 'error',

    // 如果默认的 defaultProps 跟 propTypes 定义的类型不一致，报错
    'react/default-props-match-prop-types': 'error',
    // 允许省略 displayName
    'react/display-name': 'off',
    // 当使用组件的时候，允许省略组件的 props
    'react/forbid-component-props': 'off',
    'react/forbid-elements': 'off',
    // 禁止使用其它组件的 propTypes
    'react/forbid-foreign-prop-types': 'warn',
    'react/forbid-prop-types': 'off',
    // 对于 boolean 类型的属性值，必须保持一致
    'react/jsx-boolean-value': 'error',
    // jsx 闭合括号的位置必须在新的一行
    'react/jsx-closing-bracket-location': 'error',
    // jsx 属性和表达式中禁止花括号内的空格
    'react/jsx-curly-spacing': 'error',
    // jsx 属性的等号前后禁止空格
    'react/jsx-equals-spacing': 'error',
    // jsx 文件只允许 js, jsx, es 的后缀
    'react/jsx-filename-extension': [
      'error',
      { extensions: ['.js', '.jsx', '.es'] }
    ],
    // jsx 的属性必须换行
    'react/jsx-first-prop-new-line': 'error',
    'react/jsx-handler-names': 'off',
    // jsx 的属性缩进 2 个空格
    'react/jsx-indent-props': ['error', 2],
    // jsx 的缩进 2 个空格
    'react/jsx-indent': ['error', 2],
    // jsx 检查缺失 key 的情况
    'react/jsx-key': 'error',
    // jsx 一行不允许多个属性
    'react/jsx-max-props-per-line': [
      'error',
      { when: 'multiline', maximum: 1 }
    ],
    'react/jsx-no-bind': 'off',
    // 文本节点如果有 // 或者 /* 的时候，是不允许的
    'react/jsx-no-comment-textnodes': 'warn',
    // jsx 禁止重复的属性
    'react/jsx-no-duplicate-props': 'error',
    'react/jsx-no-literals': 'off',
    // 禁止使用 target="_blank"
    'react/jsx-no-target-blank': 'error',
    'react/jsx-no-undef': 'error',
    'react/jsx-pascal-case': 'error',
    'react/jsx-sort-props': 'off',
    'react/jsx-tag-spacing': 'off',
    'react/jsx-uses-react': 'off',
    'react/jsx-uses-vars': 'error',
    'react/jsx-wrap-multilines': 'error',
    'react/no-array-index-key': 'error',
    'react/no-children-prop': 'error',
    'react/no-danger-with-children': 'error',
    'react/no-danger': 'warn',
    'react/no-deprecated': 'error',
    // TODO: 当前下面2个规则会在异步函数里都报错，等官方解决后再打开
    //
    // https://github.com/yannickcr/eslint-plugin-react/issues/1110
    'react/no-did-mount-set-state': 'off',
    'react/no-did-update-set-state': 'off',
    'react/no-direct-mutation-state': 'error',
    'react/no-find-dom-node': 'error',
    'react/no-is-mounted': 'error',
    'react/no-multi-comp': ['error', { ignoreStateless: true }],
    'react/no-render-return-value': 'error',
    'react/no-set-state': 'off',
    'react/no-string-refs': 'error',
    'react/no-unescaped-entities': 'error',
    'react/no-unknown-property': 'error',
    'react/no-unused-prop-types': 'warn',
    'react/no-will-update-set-state': 'error',
    'react/prefer-es6-class': 'error',
    'react/prefer-stateless-function': 'error',
    'react/prop-types': 'off',
    'react/react-in-jsx-scope': 'off',
    'react/require-default-props': 'off',
    'react/require-optimization': 'off',
    'react/require-render-return': 'error',
    'react/self-closing-comp': ['error', { html: false }],
    'react/sort-prop-types': 'off',
    'react/style-prop-object': 'error',
    'react/void-dom-elements-no-children': 'error',
    // jsx 文件中属性的顺序
    'react/sort-comp': [
      0,
      {
        order: [
          'static-properties',
          'static-methods',
          'state',
          'properties',
          'lifecycle',
          'everything-else',
          'render'
        ],
        groups: {
          'static-properties': [
            'displayName',
            'propTypes',
            'contextTypes',
            'childContextTypes',
            'mixins',
            'statics'
          ],
          lifecycle: [
            'getDefaultProps',
            'getInitialState',
            'constructor',
            'getChildContext',
            'componentWillMount',
            'componentDidMount',
            'componentWillReceiveProps',
            'shouldComponentUpdate',
            'componentWillUpdate',
            'componentDidUpdate',
            'componentDidCatch',
            'componentWillUnmount'
          ]
        }
      }
    ]
  }
};
